"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDatasetProfiles = exports.generateDynamicDataset = exports.getDatasourceProfiles = void 0;
const cals_1 = require("@cloudbase/cals");
const generate_1 = require("../generator/core/generate");
const config_1 = require("../generator/config");
const index_1 = require("./index");
/**
 * 将对象的key的首字母转换为小写
 * { UserName: 'xxx', Id: 'xxxx'} => { userName: 'xxx', id: 'xxx' }
 * 默认只处理最外层, 若 deep 为 true, 则递归处理
 */
function lowercaseKey(obj, deep) {
    if (!obj || typeof obj !== 'object')
        return obj;
    if (Array.isArray(obj))
        return obj.map((item) => lowercaseKey(item, deep));
    const result = {};
    return Object.keys(obj).reduce((acc, key) => {
        const smallCase = key.charAt(0).toLowerCase() + key.slice(1);
        const val = obj[key];
        acc[smallCase] = deep ? lowercaseKey(val, deep) : val;
        return acc;
    }, result);
}
/**
 * 标准化从云API获取的数据源结构
 *  1. 将大写转换为小写
 *  2. 将 methods 和 methods 从字符串转换为对象
 */
function standardizeDataSource(ds) {
    const transformed = lowercaseKey(ds, true);
    try {
        const { schema } = ds;
        if (typeof transformed.schema === 'string') {
            transformed.schema = JSON.parse(transformed.schema);
        }
        if (!transformed.schema || typeof transformed.schema !== 'object') {
            // console.warn(
            //   '[standardizeDataSource]invalid schema',
            //   schema,
            //   'in datasource',
            //   transformed
            // );
            transformed.schema = {};
        }
    }
    catch (error) {
        // console.warn(
        //   '[standardizeDataSource] unable to transform schema',
        //   transformed.schema
        // );
        transformed.schema = {};
    }
    try {
        const { methods } = ds;
        if (typeof transformed.methods === 'string') {
            transformed.methods = JSON.parse(transformed.methods);
        }
        if (!Array.isArray(transformed.methods)) {
            // console.warn(
            //   '[standardizeDataSource]invalid methods',
            //   methods,
            //   'in datasource',
            //   transformed
            // );
            transformed.methods = [];
        }
    }
    catch (error) {
        // console.warn(
        //   '[standardizeDataSource]unable to transform methods',
        //   transformed.methods
        // );
        transformed.methods = [];
    }
    return transformed;
}
/**
 * 简化数据源描述信息数组, 以供低码运行时使用
 *  该数组内容应当输出到 template/src/datasources/datasource-profiles.js.tpl 中
 * @param datasources 完整的数据源描述信息数组
 */
function getDatasourceProfiles(datasources) {
    return ((datasources === null || datasources === void 0 ? void 0 : datasources.map((item) => {
        var _a;
        const ds = standardizeDataSource(item);
        return {
            id: ds.id,
            title: ds.title,
            name: ds.name,
            type: ds.type,
            childDataSourceNames: ds.childDataSourceNames,
            schema: ds.schema,
            methods: (_a = ds.methods) === null || _a === void 0 ? void 0 : _a.map((method) => {
                return {
                    name: method.name,
                    type: method.type,
                    outParams: method.outParams,
                };
            }),
        };
    })) || []);
}
exports.getDatasourceProfiles = getDatasourceProfiles;
function generateDynamicDataset(_dataset = {}) {
    var _a;
    let { query, pageName: _, ...dataset } = _dataset;
    const { state } = dataset;
    if (state) {
        for (let key in state) {
            /**
             * 减少体积
             * 过滤运行态无用字段
             */
            let { label: _, dataType: __, ...config } = state[key];
            if (config.varType === 'datasource' && ((_a = config.initMethod) === null || _a === void 0 ? void 0 : _a.params)) {
                let { params } = config.initMethod;
                if (params) {
                    let processed = {};
                    for (let paramKey in params) {
                        let bind = params[paramKey];
                        if (!bind.type || bind.type === cals_1.PropBindType.static) {
                            let { value } = bind;
                            processed[paramKey] = `${config_1.REPLACE_SIGN}function (app, $page) {
                return (${typeof value === 'string' ? `'${value}'` : JSON.stringify(value)})
              }${config_1.REPLACE_SIGN}`;
                        }
                        else {
                            let jsExp = (0, generate_1.generateDataBind)(bind);
                            let imports = 'const $app = app;';
                            try {
                                imports += (0, index_1.generateExpressionAlias)(jsExp);
                            }
                            catch (e) { }
                            processed[paramKey] = `${config_1.REPLACE_SIGN}function (app, $page, $w) {${imports} return (\n${jsExp || 'undefined'}\n)} ${config_1.REPLACE_SIGN}`;
                        }
                    }
                    config.initMethod.params = processed;
                }
            }
        }
    }
    return dataset;
}
exports.generateDynamicDataset = generateDynamicDataset;
function getDatasetProfiles(mainAppData, apps) {
    const result = {};
    if (mainAppData.dataset) {
        result.$global = generateDynamicDataset(mainAppData.dataset);
    }
    (apps || []).forEach((app) => {
        var _a;
        (_a = app.pageInstanceList) === null || _a === void 0 ? void 0 : _a.forEach((pageInstance) => {
            let p = pageInstance;
            if (p.dataset) {
                result[app.rootPath ? `${app.rootPath}/${p.id}` : p.id] = generateDynamicDataset(p.dataset);
            }
        });
    });
    return result;
}
exports.getDatasetProfiles = getDatasetProfiles;
